package com.cisco.cuic.api.examples;

import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import com.cisco.cuic.api.client.APICustomWorkflowDefinitionList;
import com.cisco.cuic.api.client.APILoginProfile;
import com.cisco.cuic.api.client.APIReportDefinition;
import com.cisco.cuic.api.client.APITabularReport;
import com.cisco.cuic.api.client.CuicServer;
import com.cisco.cuic.api.client.CustomActionDefinition;
import com.cisco.cuic.api.client.IPAddressReservation;
import com.cisco.cuic.api.models.UserAPIGlobal;

public class LegacyAPIWrapperTests {

	public static void main(String[] args) throws Exception {
		CuicServer api = CuicServer.getAPI("172.29.110.250",
				// "562FDF763A384E78B9BAB7FE02CA13B6", "https", 443);
				"E085458F7D744CF3BD0B277133F4167F", "https", 443);

		// executeGetMyLoginProfile(api);
		// executeGetAllReports(api);
		// executeGetIPAddressPool(api);
		// executeDeleteUser(api);
		// executeGetWorkflows(api);
		// executeGetAllVDCs(api);
		executeExportWorkflows(api);
	}

	public static void executeExportWorkflows(CuicServer api) throws Exception {
		UserAPIGlobal global = new UserAPIGlobal(api);
		String workflowNames = "Email Task";
		APICustomWorkflowDefinitionList result = global.userAPIExportWorkflows(workflowNames);
		System.out.println(result.getWorkflowDefinitonsMap());
	}

	public static void executeGetAllVDCs(CuicServer api) {
		UserAPIGlobal global = new UserAPIGlobal(api);
		APITabularReport r = null;
		try {
			r = global.userAPIGetAllVDCs();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		List<Map<String, Object>> listofMaps = r.getRows();
		System.out.println("rows " + r.getRowCount());

		// Map<String,Object> map = null;
		for (Map<String, Object> map : listofMaps) {
			for (Entry<String, Object> entry : map.entrySet()) {
				System.out.println(entry.getKey() + " " + entry.getValue().toString());
			}
		}
	}

	public static void executeGetWorkflows(CuicServer api) {
		UserAPIGlobal global = new UserAPIGlobal(api);
		CustomActionDefinition[] def = null;

		try {
			def = global.userAPIGetWorkflows("EMC");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		for (CustomActionDefinition actionDef : def) {
			System.out.println(actionDef.getName() + " " + actionDef.getDescription());
		}
	}

	public static void executeDeleteUser(CuicServer api) {
		UserAPIGlobal global = new UserAPIGlobal(api);
		boolean deleted = false;

		try {
			deleted = global.userAPIDeleteUser("mytestuser");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		System.out.println("Deleted " + deleted);
	}

	public static void executeGetUserLoginProfile(CuicServer api) {
		UserAPIGlobal global = new UserAPIGlobal(api);

		APILoginProfile reportDef = null;
		try {
			reportDef = global.userAPIGetUserLoginProfile("admin");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		System.out.println(reportDef.getFirstName() + " " + reportDef.getUserId());
	}

	public static void executeGetIPAddressPool(CuicServer api) {
		UserAPIGlobal global = new UserAPIGlobal(api);

		List<IPAddressReservation> reportDef = null;
		try {
			reportDef = global.userAPIgetIPAddressPool();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		for (IPAddressReservation ip : reportDef) {
			System.out.println(ip.getAccountName() + " " + ip.getIpAddress());
		}
	}

	public static void executeGetAllReports(CuicServer api) {
		UserAPIGlobal global = new UserAPIGlobal(api);

		List<APIReportDefinition> reportDef = null;
		try {
			reportDef = global.userAPIGetAvailableReports("datacenter", "");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		for (APIReportDefinition def : reportDef) {
			System.out.println(def.getReportLabel() + " " + def.getReportId() + " " + def.getReportType());
		}

	}

	public static void executeGetMyLoginProfile(CuicServer api) {

		UserAPIGlobal global = new UserAPIGlobal(api);
		APILoginProfile profile = null;
		try {
			profile = global.userAPIGetMyLoginProfile();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		if (profile != null) {
			System.out.println(profile.getRole());
		} else {
			System.out.println("profile can't be retrieved");
		}
	}
}
