package com.cisco.cuic.api.examples;

import static org.junit.Assert.*;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import junit.framework.Assert;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.cloupia.sdk.api.CloupiaClient;
import com.cloupia.sdk.api.RemoteServiceException;
import com.cloupia.sdk.util.JsonUtil;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;

public class TestRestAPIV1Reports {

	private static final String REST_SERVER_PROPERTIES = "rest-server.properties";
	private static final String UCSM_CONTEXT_NAME = "ucsm";
	private Properties props;
	private CloupiaClient client;
	private Map<String, String> reportLabelIdMap;
	private JsonObject obj;
	private JsonParser parser;

	@Before
	public void setUp() throws Exception {
		System.out.println("current dir : " + System.getProperty("user.dir"));
		// InputStream is =
		// this.getClass().getResourceAsStream(REST_SERVER_PROPERTIES);
		// //REST_SERVER_PROPERTIES is a constant = "rest-server.properties"
		InputStream is = new FileInputStream(new File(REST_SERVER_PROPERTIES));
		props = new Properties();
		props.load(is);
		client = new CloupiaClient(props.getProperty("server-address"),
				Integer.parseInt(props.getProperty("server-port", "443")), props.getProperty("rest-api-access-key"));
		parser = new JsonParser();
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void getAvailableReportsForUCSMContext() throws Exception {
		// param0 is contextName, param1 is empty
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetAvailableReports",
				Arrays.asList(new String[] { UCSM_CONTEXT_NAME, "" })); // UCSM_CONTEXT_NAME
																		// ="ucsm";
		System.out.println("json response for available reports for UCSM context : " + jsonResponse);
		JsonArray array = jsonResponse.getAsJsonArray();
		reportLabelIdMap = new HashMap<String, String>();

		for (JsonElement ele : array) {
			obj = ele.getAsJsonObject();
			if (obj.get("reportType").getAsString().equals("tabular"))
				reportLabelIdMap.put(obj.get("reportLabel").getAsString(), obj.get("reportId").getAsString());
		}

		System.out.println("report label to id mapping : " + reportLabelIdMap.toString());
		for (String reportId : reportLabelIdMap.values()) {
			// specify a UCSM account instance added in CUIC here
			this.getUcsmTabularReport("UCSM-VNX300", reportId);
			// getTabularReport(UCSM_CONTEXT_NAME, "ucsm-34-good", reportId);
			// getTabularReportMetadata(UCSM_CONTEXT_NAME, "ucsm-34-good",
			// reportId);
		}
	}

	private void getTabularReport(String contextName, String accountName, String reportId) throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetTabularReport",
				Arrays.asList(new String[] { contextName, accountName, reportId }));
		System.out.println("tabular report for context : " + contextName + " accountName : " + accountName
				+ " report ID " + reportId + " is ");
		System.out.println(jsonResponse.toString() + "\n");
	}

	private void getTabularReportMetadata(String contextName, String accountName, String reportId) throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetTabularReportMetadata",
				Arrays.asList(new String[] { contextName, accountName, reportId }));
		System.out.println("tabular report metadata for context : " + contextName + " accountName : " + accountName
				+ " report ID " + reportId + " is ");
		System.out.println(jsonResponse.toString() + "\n");
	}

	@Test
	public void getAvailableReportsForNetAppFilerContext() throws Exception {
		// param0 is contextName, param1 is empty
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetAvailableReports",
				Arrays.asList(new String[] { "netapp_filer", "" }));
		System.out.println("json response for available reports for netapp filer context :  " + jsonResponse);
		JsonArray array = jsonResponse.getAsJsonArray();
		reportLabelIdMap = new HashMap<String, String>();

		for (JsonElement ele : array) {
			obj = ele.getAsJsonObject();
			if (obj.get("reportType").getAsString().equals("tabular"))
				reportLabelIdMap.put(obj.get("reportLabel").getAsString(), obj.get("reportId").getAsString());
		}

		System.out.println("report label to id mapping : " + reportLabelIdMap.toString());

	}

	@Test
	public void getApplicableManagedReports() throws Exception {
		System.out.println(
				"applicable managed reports " + client.sendJSONRequest("userAPIGetApplicableManagedReports", null));
		// System.out.println("per vm report : " + jsonResponse.toString());
	}

	@Test
	public void getPerCloudTabularReport() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetTabularReport",
				Arrays.asList(new String[] { "cloud", "cloud250", "VMS-T0" }));
		String val1 = jsonResponse.toString();

		JsonElement jsonResponse2 = client.sendJSONRequest("userAPIGetTabularReport",
				Arrays.asList(new String[] { "1", "cloud250", "VMS-T0" }));

		String val2 = jsonResponse2.toString();

		Assert.assertEquals(val1, val2);

		// System.out.println("tabular report for context : "+contextName + "
		// accountName : "+ accountName + " report ID "+ reportId + " is ");
		System.out.println(jsonResponse.toString() + "\n");
	}

	public void getUcsmTabularReport(String contextValue, String reportId) throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetTabularReport",
				Arrays.asList(new String[] { "ucsm", contextValue, reportId }));
		String val1 = jsonResponse.toString();

		JsonElement jsonResponse2 = client.sendJSONRequest("userAPIGetTabularReport",
				Arrays.asList(new String[] { "37", contextValue, reportId }));

		String val2 = jsonResponse2.toString();

		Assert.assertEquals(val1, val2);

		// System.out.println("tabular report for context : "+contextName + "
		// accountName : "+ accountName + " report ID "+ reportId + " is ");
		System.out.println("report json string for ucsm context " + contextValue + " with reportId : " + reportId + " "
				+ jsonResponse.toString() + "\n");
	}

	@Test
	public void getPerVMTabularReport() throws Exception {
		System.out.println(client.sendJSONRequest("userAPIGetTabularReport",
				Arrays.asList(new String[] { "vm", "2319", "DISKS-T0" })));
		// System.out.println("per vm report : " + jsonResponse.toString());
	}

	@Test
	public void getAvailableReportsForCloudContext() throws Exception {
		// param0 is contextName, param1 is empty
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetAvailableReports",
				Arrays.asList(new String[] { "cloud", "cloud250" }));
		System.out.println("json response for available reports for cloud context :  " + jsonResponse);
		JsonArray array = jsonResponse.getAsJsonArray();
		reportLabelIdMap = new HashMap<String, String>();

		for (JsonElement ele : array) {
			obj = ele.getAsJsonObject();
			if (obj.get("reportType").getAsString().equals("tabular"))
				reportLabelIdMap.put(obj.get("reportLabel").getAsString(), obj.get("reportId").getAsString());
		}

		System.out.println("report label to id mapping : " + reportLabelIdMap.toString());

	}

	@Test
	public void getAllCatalogs() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetAllCatalogs", null);
		System.out.println("json response for all catalogs" + jsonResponse);
		Assert.assertEquals(3, jsonResponse.getAsJsonObject().get("rows").getAsJsonArray().size());

	}

	@Test
	public void getMyLoginProfile() throws Exception {

		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetMyLoginProfile", null);
		System.out.println("json response for my login profile " + jsonResponse);
	}

	@Test
	public void getUserAPIGetAllVMs() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetAllVMs", null);// Arrays.asList(new
																					// String[]
																					// {""}));
		System.out.println("all VM's for the logged in user: " + jsonResponse);
		JsonArray array = jsonResponse.getAsJsonObject().get("rows").getAsJsonArray();
		JsonObject vmObj;
		List<String> vm_ids = new ArrayList<String>();
		for (JsonElement ele : array) {
			vmObj = ele.getAsJsonObject();
			vm_ids.add(vmObj.get("VM_ID").getAsString());
		}
		System.out.println("list of available VM ID's " + vm_ids);

	}

	@Test
	public void getUserAPIGetAllVDCs() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetAllVDCs", null);// Arrays.asList(new
																						// String[]
																						// {""}));
		System.out.println("all VDCs for logged in user group : " + jsonResponse);
		Assert.assertEquals(3, jsonResponse.getAsJsonObject().get("rows").getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetServiceRequests() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetServiceRequests", null);// Arrays.asList(new
																								// String[]
																								// {""}));
		System.out.println("all service request for user group : " + jsonResponse);
		Assert.assertEquals(57, jsonResponse.getAsJsonObject().get("rows").getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetVMSummary() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetVMSummary",
				Arrays.asList(new String[] { "2293" }));
		System.out.println("VM summary : " + jsonResponse);
		Assert.assertEquals(1, jsonResponse.getAsJsonObject().get("rows").getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetVMSnapshotDetails() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetVMSnapshotDetails",
				Arrays.asList(new String[] { "2293" }));
		System.out.println("VM snapshot details : " + jsonResponse);
		Assert.assertEquals(4, jsonResponse.getAsJsonObject().get("rows").getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetAvailableVMActions() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetAvailableVMActions",
				Arrays.asList(new String[] { "2293" }));
		System.out.println("VM actions : " + jsonResponse);
		Assert.assertEquals(19, jsonResponse.getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetWorkflows() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetWorkflows",
				Arrays.asList(new String[] { "API Workflows" }));
		System.out.println("API workflows : " + jsonResponse);
		Assert.assertEquals(0, jsonResponse.getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetInstantDataReport() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetInstantDataReport",
				Arrays.asList(new String[] { "cloud", "cloud250", "TOTAL-CAPACITY-PER-STORAGE-TYPE-S1" }));
		System.out.println("API snashot report : " + jsonResponse);
		// Assert.assertEquals(0,jsonResponse.getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetSnapshotReport() throws Exception {
		try {
			JsonElement jsonResponse = client.sendJSONRequest("userAPIGetSnapshotReport",
					Arrays.asList(new String[] { "cloud", "cloud250", "TOTAL-CAPACITY-PER-STORAGE-TYPE-S1" }));
			System.out.println("API snashot report : " + jsonResponse);

		} catch (RemoteServiceException rse) {
			System.out.println("userAPIGetSnapshotReport error : " + rse.getMessage());
		}

	}

	@Test
	public void getUserAPIGetSystemTasksReport() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetSystemTasksReport", null);
		System.out.println("API system tasks report : " + jsonResponse);
		// Assert.assertEquals(0,jsonResponse.getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetSystemResourceUsageReport() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetSystemResourceUsageReport", null);
		System.out.println("API system resource usage report : " + JsonUtil.prettyPrint(jsonResponse));
		// Assert.assertEquals(0,jsonResponse.getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetComputeAccountListReport() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetComputeAccountListReport", null);
		System.out.println("API compute account list report  : " + jsonResponse);
		// Assert.assertEquals(0,jsonResponse.getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetCloudsListReport() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetCloudsListReport", null);
		System.out.println("API cloud list report : " + jsonResponse);
		// Assert.assertEquals(0,jsonResponse.getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetNetworkDeviceListReport() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetNetworkDevicesListReport", null);
		System.out.println("API network device list report : " + jsonResponse);
		// Assert.assertEquals(0,jsonResponse.getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetStorageAccountListReport() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetStorageAccountListReport", null);
		System.out.println("API storage account list report : " + jsonResponse);
		// Assert.assertEquals(0,jsonResponse.getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetLicenseListReport() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetLicenseListReport", null);
		System.out.println("API license list report : " + jsonResponse);
		// Assert.assertEquals(0,jsonResponse.getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetComputeServerListReport() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetComputeServerListReport", null);
		System.out.println("API compute server list report : " + jsonResponse);
		// Assert.assertEquals(0,jsonResponse.getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetCloudPollingHistoryReport() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetCloudPollingHistoryReport", null);
		System.out.println("API cloud polling history report : " + jsonResponse);
		// Assert.assertEquals(0,jsonResponse.getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetAllContexts() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetAllContexts", null);
		System.out.println("API get all contexts : " + jsonResponse);
		// Assert.assertEquals(0,jsonResponse.getAsJsonArray().size());

	}

	@Test
	public void getUserAPIGetHistoricalReportForNumberOfVMs() throws Exception {
		// JsonElement jsonResponse =
		// client.sendJSONRequest("userAPIGetHistoricalReport",
		// Arrays.asList(new String[]
		// {"cloud","cloud250","TREND-NUMBER-OF-VMS-H0","daily"}));
		try {
			JsonElement jsonResponse = client.sendJSONRequest("userAPIGetHistoricalReport",
					Arrays.asList(new String[] { "vm", "2319", "TREND-CPU-USAGE-(PERCENT)-H0", "daily" }));
			System.out.println("historical /trend report for CPU usage for VM daily : " + jsonResponse);
		} catch (Exception ex) {
			fail(ex.getMessage());
		}

	}

	@Test
	public void getUserAPIGetHistoricalReportForStorageCapacity() throws Exception {
		try {
			JsonElement jsonResponse = client.sendJSONRequest("userAPIGetHistoricalReport", Arrays.asList(new String[] {
					"netapp_filer", "ontap-remote", "TREND-STORAGE-CAPACITY,-USED-&-FREE-H1", "daily" }));
			System.out.println("API snashot report : " + jsonResponse);
		} catch (Exception ex) {
			fail(ex.getMessage());
		}

	}

	@Test
	public void getUserAPIGetAvailableVMAction() throws Exception {
		JsonElement jsonResponse = client.sendJSONRequest("userAPIGetAvailableVMActions",
				Arrays.asList(new String[] { "2293" }));
		System.out.println("VM actions : " + jsonResponse);
		// Assert.assertEquals(1,jsonResponse.getAsJsonArray().size());

	}

}
